#  Copyright (c) 2016 Jeroen Wouters

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np

##################
#
# Helper functions/classes
#
##################

class TreeNode():
	def __init__(self, data = None):
		self.children = []
		self.parent = None
		self.data = data
		
	def appendChild(self, child):
		self.children.append(child)
		child.parent = self
		
	def path(self):
		# returns the path starting from the node to the root of the tree
		path = [self]
		root = self
		while root.parent != None:
			root = root.parent
			path.append(root)
		path.reverse()
		return path

def discrete_random(values, probabilities, size):
    # Generate discrete random variable with prescribed probabilities
    # values has to be a numpy array
    # Samples uniformly distributed [0,1] floats and transforms to integers by
    # checking in which bin they fall (np.digitize)
    # http://stackoverflow.com/questions/11373192/generating-discrete-random-variables-with-specified-weights-using-scipy-or-numpy
    bins = np.cumsum(probabilities)
    return values[np.digitize(np.random.random_sample(size), bins)]
